@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Utils::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Utils::VERSION   = '1.007_000';
+	$Type::Utils::VERSION   = '1.007_001';
 }
 
 $Type::Utils::VERSION =~ tr/_//d;
@@ -80,6 +80,18 @@ sub extends
 				);
 			}
 		}
+		elsif ($lib->isa('Specio::Exporter'))
+		{
+			my $types = $lib->Specio::Registry::exportable_types_for_package;
+			for my $name (sort keys %$types)
+			{
+				my $specio = $types->{$name};
+				my $tt     = Types::TypeTiny::to_TypeTiny($specio);
+				$caller->add_type(
+					$tt->create_child_type(library => $caller, name => $name)
+				);
+			}
+		}
 		else
 		{
 			_croak("'$lib' is not a type constraint library");
@@ -750,7 +762,7 @@ significant performance improvements.
    
    warn EvenInt->inline_check('$xxx');  # demonstration
 
-B<Experimental:> your C<inline_as> block can return a list, in which case
+Your C<inline_as> block can return a list, in which case
 these will be smushed together with "&&". The first item on the list may
 be undef, in which case the undef will be replaced by the inlined parent
 type constraint. (And will throw an exception if there is no parent.)
@@ -762,10 +774,6 @@ type constraint. (And will throw an exception if there is no parent.)
          return (undef, "($_ % 2 == 0)");
       };
 
-Returning a list like this is considered experimental, is not tested very
-much, and I offer no guarantees that it will necessarily work with
-Moose/Mouse/Moo.
-
 =item C<< class_type $name, { class => $package, %options } >>
 
 =item C<< class_type { class => $package, %options } >>
