@@ -12,7 +12,7 @@ BEGIN {
 
 BEGIN {
 	$Types::Standard::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::VERSION   = '1.007_000';
+	$Types::Standard::VERSION   = '1.007_001';
 }
 
 $Types::Standard::VERSION =~ tr/_//d;
@@ -31,6 +31,7 @@ BEGIN {
 	$is_class_loaded = q{sub {
 		return !!0 if ref $_[0];
 		return !!0 if not $_[0];
+		return !!0 if ref(do { my $tmpstr = $_[0]; \$tmpstr }) ne 'SCALAR';
 		my $stash = do { no strict 'refs'; \%{"$_[0]\::"} };
 		return !!1 if exists $stash->{'ISA'};
 		return !!1 if exists $stash->{'VERSION'};
@@ -679,12 +680,12 @@ $meta->add_type({
 	name       => "Tied",
 	parent     => $_ref,
 	constraint => sub {
-		!!tied(Scalar::Util::reftype($_) eq 'HASH' ?  %{$_} : Scalar::Util::reftype($_) eq 'ARRAY' ?  @{$_} :  ${$_})
+		!!tied(Scalar::Util::reftype($_) eq 'HASH' ?  %{$_} : Scalar::Util::reftype($_) eq 'ARRAY' ?  @{$_} : Scalar::Util::reftype($_) =~ /^(SCALAR|REF)$/ ?  ${$_} :  undef)
 	},
 	inlined    => sub {
 		my ($self, $var) = @_;
 		$self->parent->inline_check($var)
-		. " and !!tied(Scalar::Util::reftype($var) eq 'HASH' ? \%{$var} : Scalar::Util::reftype($var) eq 'ARRAY' ? \@{$var} : \${$var})"
+		. " and !!tied(Scalar::Util::reftype($var) eq 'HASH' ? \%{$var} : Scalar::Util::reftype($var) eq 'ARRAY' ? \@{$var} : Scalar::Util::reftype($var) =~ /^(SCALAR|REF)\$/ ? \${$var} : undef)"
 	},
 	name_generator => sub
 	{
